@@ -26,23 +26,29 @@
 #include "bino.hpp"
 #include "log.hpp"
 #include "tools.hpp"
+#include "digestiblemedia.hpp"
 #include "metadata.hpp"
 
 
 static Bino* binoSingleton = nullptr;
 
-Bino::Bino(const Screen& screen, bool swapEyes) :
+Bino::Bino(ScreenType screenType, const Screen& screen, bool swapEyes) :
     _wantExit(false),
     _videoSink(nullptr),
     _audioOutput(nullptr),
     _player(nullptr),
     _audioInput(nullptr),
     _videoInput(nullptr),
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
     _screenInput(nullptr),
+#endif
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 6, 0))
     _windowInput(nullptr),
+#endif
     _captureSession(nullptr),
     _lastFrameInputMode(Input_Unknown),
     _lastFrameSurroundMode(Surround_Unknown),
+    _screenType(screenType),
     _screen(screen),
     _frameIsNew(false),
     _frameWasSerialized(true),
@@ -59,8 +65,12 @@ Bino::~Bino()
     delete _player;
     delete _audioInput;
     delete _videoInput;
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
     delete _screenInput;
+#endif
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 6, 0))
     delete _windowInput;
+#endif
     delete _captureSession;
     binoSingleton = nullptr;
 }
@@ -150,6 +160,7 @@ void Bino::startCaptureModeCamera(
     emit stateChanged();
 }
 
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
 void Bino::startCaptureModeScreen(
         bool withAudioInput,
         const QAudioDevice& audioInputDevice,
@@ -162,7 +173,9 @@ void Bino::startCaptureModeScreen(
     _screenInput->setActive(true);
     emit stateChanged();
 }
+#endif
 
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 6, 0))
 void Bino::startCaptureModeWindow(
         bool withAudioInput,
         const QAudioDevice& audioInputDevice,
@@ -175,6 +188,7 @@ void Bino::startCaptureModeWindow(
     _windowInput->setActive(true);
     emit stateChanged();
 }
+#endif
 
 void Bino::stopCaptureMode()
 {
@@ -183,10 +197,14 @@ void Bino::stopCaptureMode()
         _captureSession = nullptr;
         delete _videoInput;
         _videoInput = nullptr;
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
         delete _screenInput;
         _screenInput = nullptr;
+#endif
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 6, 0))
         delete _windowInput;
         _windowInput = nullptr;
+#endif
         if (_audioInput) {
             delete _audioInput;
             _audioInput = nullptr;
@@ -212,16 +230,13 @@ void Bino::mediaChanged(PlaylistEntry entry)
     if (entry.noMedia()) {
         _player->stop();
     } else {
-        // QMediaPlayer does not work with simply setting a new source via setSource().
-        // Apparently we at least need to flush all buffers with setSource(QUrl()) first.
-        // But that triggers playbackstateChanged() events which mess up our playlist.
-        // To work around this problem, we use the big hammer and destroy and recreate
-        // the QMediaPlayer before setting the new URL. Not exactly elegant...
-        stopPlaylistMode();
-        startPlaylistMode();
-        _player->setSource(entry.url);
+        // Get meta data
         MetaData metaData;
         metaData.detectCached(entry.url);
+        // Special handling of files that cannot be digested by QtMultimedia directly
+        QUrl digestibleUrl = digestibleMediaUrl(entry.url);
+        // Set new source
+        _player->setSource(digestibleUrl);
         if (entry.videoTrack >= 0) {
             _player->setActiveVideoTrack(entry.videoTrack);
         }
@@ -499,12 +514,12 @@ SurroundMode Bino::assumeSurroundMode() const
 
 void Bino::serializeStaticData(QDataStream& ds) const
 {
-    ds << _screen;
+    ds << _screenType << _screen;
 }
 
 void Bino::deserializeStaticData(QDataStream& ds)
 {
-    ds >> _screen;
+    ds >> _screenType >> _screen;
 }
 
 void Bino::serializeDynamicData(QDataStream& ds)
@@ -543,9 +558,8 @@ bool Bino::wantExit() const
 
 bool Bino::initProcess()
 {
-    bool isGLES = QOpenGLContext::currentContext()->isOpenGLES();
     bool haveAnisotropicFiltering = checkTextureAnisotropicFilterAvailability();
-    LOG_DEBUG("Using OpenGL in the %s variant", isGLES ? "ES" : "Desktop");
+    LOG_DEBUG("Using OpenGL in the %s variant", IsOpenGLES ? "ES" : "Desktop");
 
     // Qt-based OpenGL initialization
     initializeOpenGLFunctions();
@@ -731,23 +745,20 @@ bool Bino::initProcess()
     // Screen geometry
     glGenVertexArrays(1, &_screenVao);
     glBindVertexArray(_screenVao);
-    GLuint positionBuf;
-    glGenBuffers(1, &positionBuf);
-    glBindBuffer(GL_ARRAY_BUFFER, positionBuf);
+    glGenBuffers(1, &_positionBuf);
+    glBindBuffer(GL_ARRAY_BUFFER, _positionBuf);
     glBufferData(GL_ARRAY_BUFFER, _screen.positions.size() * sizeof(float),
             _screen.positions.constData(), GL_STATIC_DRAW);
     glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 0, 0);
     glEnableVertexAttribArray(0);
-    GLuint texcoordBuf;
-    glGenBuffers(1, &texcoordBuf);
-    glBindBuffer(GL_ARRAY_BUFFER, texcoordBuf);
+    glGenBuffers(1, &_texcoordBuf);
+    glBindBuffer(GL_ARRAY_BUFFER, _texcoordBuf);
     glBufferData(GL_ARRAY_BUFFER, _screen.texcoords.size() * sizeof(float),
             _screen.texcoords.constData(), GL_STATIC_DRAW);
     glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, 0, 0);
     glEnableVertexAttribArray(1);
-    GLuint indexBuf;
-    glGenBuffers(1, &indexBuf);
-    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexBuf);
+    glGenBuffers(1, &_indexBuf);
+    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, _indexBuf);
     glBufferData(GL_ELEMENT_ARRAY_BUFFER,
             _screen.indices.length() * sizeof(unsigned int),
             _screen.indices.constData(), GL_STATIC_DRAW);
@@ -766,15 +777,14 @@ void Bino::rebuildColorPrgIfNecessary(int planeFormat, bool yuvValueRangeSmall,
 
     LOG_DEBUG("rebuilding color conversion program for plane format %d, value range %s, yuv space %s",
             planeFormat, yuvValueRangeSmall ? "small" : "full", yuvSpace ? "true" : "false");
-    bool isGLES = QOpenGLContext::currentContext()->isOpenGLES();
     QString colorVS = readFile(":src/shader-color.vert.glsl");
     QString colorFS = readFile(":src/shader-color.frag.glsl");
     colorFS.replace("$PLANE_FORMAT", QString::number(planeFormat));
     colorFS.replace("$VALUE_RANGE_SMALL", yuvValueRangeSmall ? "true" : "false");
     colorFS.replace("$YUV_SPACE", QString::number(yuvSpace));
-    if (isGLES) {
-        colorVS.prepend("#version 320 es\n");
-        colorFS.prepend("#version 320 es\n"
+    if (IsOpenGLES) {
+        colorVS.prepend("#version 300 es\n");
+        colorFS.prepend("#version 300 es\n"
                 "precision mediump float;\n");
     } else {
         colorVS.prepend("#version 330\n");
@@ -798,7 +808,6 @@ void Bino::rebuildViewPrgIfNecessary(SurroundMode surroundMode, bool nonLinearOu
 
     LOG_DEBUG("rebuilding view program for surround mode %s, non linear output %s",
             surroundModeToString(surroundMode), nonLinearOutput ? "true" : "false");
-    bool isGLES = QOpenGLContext::currentContext()->isOpenGLES();
     QString viewVS = readFile(":src/shader-view.vert.glsl");
     QString viewFS = readFile(":src/shader-view.frag.glsl");
     viewFS.replace("$SURROUND_DEGREES",
@@ -806,9 +815,9 @@ void Bino::rebuildViewPrgIfNecessary(SurroundMode surroundMode, bool nonLinearOu
             : surroundMode == Surround_180 ? "180"
             : "0");
     viewFS.replace("$NONLINEAR_OUTPUT", nonLinearOutput ? "true" : "false");
-    if (isGLES) {
-        viewVS.prepend("#version 320 es\n");
-        viewFS.prepend("#version 320 es\n"
+    if (IsOpenGLES) {
+        viewVS.prepend("#version 300 es\n");
+        viewFS.prepend("#version 300 es\n"
                 "precision mediump float;\n");
     } else {
         viewVS.prepend("#version 330\n");
@@ -883,10 +892,21 @@ bool Bino::drawSubtitleToImage(int w, int h, const QString& string)
     return true;
 }
 
+static int alignmentFromBytesPerLine(const void* data, int bpl)
+{
+    int alignment = 1;
+    if (uint64_t(data) % 8 == 0 && bpl % 8 == 0)
+        alignment = 8;
+    else if (uint64_t(data) % 4 == 0 && bpl % 4 == 0)
+        alignment = 4;
+    else if (uint64_t(data) % 2 == 0 && bpl % 2 == 0)
+        alignment = 2;
+    LOG_FIREHOSE("convertFrameToTexture: alignment is %d (from data %p, bpl %d)", alignment, data, bpl);
+    return alignment;
+}
+
 void Bino::convertFrameToTexture(const VideoFrame& frame, unsigned int frameTex)
 {
-    bool isGLES = QOpenGLContext::currentContext()->isOpenGLES();
-
     // 1. Get the frame data into plane textures
     int w = frame.width;
     int h = frame.height;
@@ -899,6 +919,9 @@ void Bino::convertFrameToTexture(const VideoFrame& frame, unsigned int frameTex)
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_SWIZZLE_B, GL_BLUE);
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_SWIZZLE_A, GL_ALPHA);
     if (frame.storage == VideoFrame::Storage_Image) {
+        LOG_FIREHOSE("convertFrameToTexture: format is image");
+        glPixelStorei(GL_UNPACK_ALIGNMENT, alignmentFromBytesPerLine(frame.image.constBits(), frame.image.bytesPerLine()));
+        glPixelStorei(GL_UNPACK_ROW_LENGTH, frame.image.bytesPerLine() / 4);
         glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, frame.image.constBits());
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_SWIZZLE_R, GL_BLUE);
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_SWIZZLE_G, GL_GREEN);
@@ -916,6 +939,9 @@ void Bino::convertFrameToTexture(const VideoFrame& frame, unsigned int frameTex)
         if (frame.pixelFormat == QVideoFrameFormat::Format_ARGB8888
                 || frame.pixelFormat == QVideoFrameFormat::Format_ARGB8888_Premultiplied
                 || frame.pixelFormat == QVideoFrameFormat::Format_XRGB8888) {
+            LOG_FIREHOSE("convertFrameToTexture: format argb8888");
+            glPixelStorei(GL_UNPACK_ALIGNMENT, alignmentFromBytesPerLine(planeData[0], frame.qframe.bytesPerLine(0)));
+            glPixelStorei(GL_UNPACK_ROW_LENGTH, frame.qframe.bytesPerLine(0) / 4);
             glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, planeData[0]);
             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_SWIZZLE_R, GL_ALPHA);
             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_SWIZZLE_G, GL_RED);
@@ -926,6 +952,9 @@ void Bino::convertFrameToTexture(const VideoFrame& frame, unsigned int frameTex)
         } else if (frame.pixelFormat == QVideoFrameFormat::Format_BGRA8888
                 || frame.pixelFormat == QVideoFrameFormat::Format_BGRA8888_Premultiplied
                 || frame.pixelFormat == QVideoFrameFormat::Format_BGRX8888) {
+            LOG_FIREHOSE("convertFrameToTexture: format bgra8888");
+            glPixelStorei(GL_UNPACK_ALIGNMENT, alignmentFromBytesPerLine(planeData[0], frame.qframe.bytesPerLine(0)));
+            glPixelStorei(GL_UNPACK_ROW_LENGTH, frame.qframe.bytesPerLine(0) / 4);
             glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, planeData[0]);
             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_SWIZZLE_R, GL_BLUE);
             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_SWIZZLE_G, GL_GREEN);
@@ -935,6 +964,9 @@ void Bino::convertFrameToTexture(const VideoFrame& frame, unsigned int frameTex)
             planeCount = 1;
         } else if (frame.pixelFormat == QVideoFrameFormat::Format_ABGR8888
                 || frame.pixelFormat == QVideoFrameFormat::Format_XBGR8888) {
+            LOG_FIREHOSE("convertFrameToTexture: format abgr8888");
+            glPixelStorei(GL_UNPACK_ALIGNMENT, alignmentFromBytesPerLine(planeData[0], frame.qframe.bytesPerLine(0)));
+            glPixelStorei(GL_UNPACK_ROW_LENGTH, frame.qframe.bytesPerLine(0) / 4);
             glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, planeData[0]);
             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_SWIZZLE_R, GL_ALPHA);
             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_SWIZZLE_G, GL_BLUE);
@@ -944,51 +976,91 @@ void Bino::convertFrameToTexture(const VideoFrame& frame, unsigned int frameTex)
             planeCount = 1;
         } else if (frame.pixelFormat == QVideoFrameFormat::Format_RGBA8888
                 || frame.pixelFormat == QVideoFrameFormat::Format_RGBX8888) {
+            LOG_FIREHOSE("convertFrameToTexture: format rgba8888");
+            glPixelStorei(GL_UNPACK_ALIGNMENT, alignmentFromBytesPerLine(planeData[0], frame.qframe.bytesPerLine(0)));
+            glPixelStorei(GL_UNPACK_ROW_LENGTH, frame.qframe.bytesPerLine(0) / 4);
             glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, planeData[0]);
             planeFormat = 1;
             planeCount = 1;
         } else if (frame.pixelFormat == QVideoFrameFormat::Format_YUV420P) {
+            LOG_FIREHOSE("convertFrameToTexture: format yuv420p");
+            glPixelStorei(GL_UNPACK_ALIGNMENT, alignmentFromBytesPerLine(planeData[0], frame.qframe.bytesPerLine(0)));
+            glPixelStorei(GL_UNPACK_ROW_LENGTH, frame.qframe.bytesPerLine(0));
             glTexImage2D(GL_TEXTURE_2D, 0, GL_R8, w, h, 0, GL_RED, GL_UNSIGNED_BYTE, planeData[0]);
             glBindTexture(GL_TEXTURE_2D, _planeTexs[1]);
+            glPixelStorei(GL_UNPACK_ALIGNMENT, alignmentFromBytesPerLine(planeData[1], frame.qframe.bytesPerLine(1)));
+            glPixelStorei(GL_UNPACK_ROW_LENGTH, frame.qframe.bytesPerLine(1));
             glTexImage2D(GL_TEXTURE_2D, 0, GL_R8, w / 2, h / 2, 0, GL_RED, GL_UNSIGNED_BYTE, planeData[1]);
             glBindTexture(GL_TEXTURE_2D, _planeTexs[2]);
+            glPixelStorei(GL_UNPACK_ALIGNMENT, alignmentFromBytesPerLine(planeData[2], frame.qframe.bytesPerLine(2)));
+            glPixelStorei(GL_UNPACK_ROW_LENGTH, frame.qframe.bytesPerLine(2));
             glTexImage2D(GL_TEXTURE_2D, 0, GL_R8, w / 2, h / 2, 0, GL_RED, GL_UNSIGNED_BYTE, planeData[2]);
             planeFormat = 2;
             planeCount = 3;
         } else if (frame.pixelFormat == QVideoFrameFormat::Format_YUV422P) {
+            LOG_FIREHOSE("convertFrameToTexture: format yuv422p");
+            glPixelStorei(GL_UNPACK_ALIGNMENT, alignmentFromBytesPerLine(planeData[0], frame.qframe.bytesPerLine(0)));
+            glPixelStorei(GL_UNPACK_ROW_LENGTH, frame.qframe.bytesPerLine(0));
             glTexImage2D(GL_TEXTURE_2D, 0, GL_R8, w, h, 0, GL_RED, GL_UNSIGNED_BYTE, planeData[0]);
             glBindTexture(GL_TEXTURE_2D, _planeTexs[1]);
+            glPixelStorei(GL_UNPACK_ALIGNMENT, alignmentFromBytesPerLine(planeData[1], frame.qframe.bytesPerLine(1)));
+            glPixelStorei(GL_UNPACK_ROW_LENGTH, frame.qframe.bytesPerLine(1));
             glTexImage2D(GL_TEXTURE_2D, 0, GL_R8, w / 2, h, 0, GL_RED, GL_UNSIGNED_BYTE, planeData[1]);
             glBindTexture(GL_TEXTURE_2D, _planeTexs[2]);
+            glPixelStorei(GL_UNPACK_ALIGNMENT, alignmentFromBytesPerLine(planeData[2], frame.qframe.bytesPerLine(2)));
+            glPixelStorei(GL_UNPACK_ROW_LENGTH, frame.qframe.bytesPerLine(2));
             glTexImage2D(GL_TEXTURE_2D, 0, GL_R8, w / 2, h, 0, GL_RED, GL_UNSIGNED_BYTE, planeData[2]);
             planeFormat = 2;
             planeCount = 3;
         } else if (frame.pixelFormat == QVideoFrameFormat::Format_YV12) {
+            LOG_FIREHOSE("convertFrameToTexture: format yv12");
+            glPixelStorei(GL_UNPACK_ALIGNMENT, alignmentFromBytesPerLine(planeData[0], frame.qframe.bytesPerLine(0)));
+            glPixelStorei(GL_UNPACK_ROW_LENGTH, frame.qframe.bytesPerLine(0));
             glTexImage2D(GL_TEXTURE_2D, 0, GL_R8, w, h, 0, GL_RED, GL_UNSIGNED_BYTE, planeData[0]);
             glBindTexture(GL_TEXTURE_2D, _planeTexs[1]);
+            glPixelStorei(GL_UNPACK_ALIGNMENT, alignmentFromBytesPerLine(planeData[1], frame.qframe.bytesPerLine(1)));
+            glPixelStorei(GL_UNPACK_ROW_LENGTH, frame.qframe.bytesPerLine(1));
             glTexImage2D(GL_TEXTURE_2D, 0, GL_R8, w / 2, h / 2, 0, GL_RED, GL_UNSIGNED_BYTE, planeData[1]);
             glBindTexture(GL_TEXTURE_2D, _planeTexs[2]);
+            glPixelStorei(GL_UNPACK_ALIGNMENT, alignmentFromBytesPerLine(planeData[2], frame.qframe.bytesPerLine(2)));
+            glPixelStorei(GL_UNPACK_ROW_LENGTH, frame.qframe.bytesPerLine(2));
             glTexImage2D(GL_TEXTURE_2D, 0, GL_R8, w / 2, h / 2, 0, GL_RED, GL_UNSIGNED_BYTE, planeData[2]);
             planeFormat = 3;
             planeCount = 3;
         } else if (frame.pixelFormat == QVideoFrameFormat::Format_NV12) {
+            LOG_FIREHOSE("convertFrameToTexture: format nv12");
+            glPixelStorei(GL_UNPACK_ALIGNMENT, alignmentFromBytesPerLine(planeData[0], frame.qframe.bytesPerLine(0)));
+            glPixelStorei(GL_UNPACK_ROW_LENGTH, frame.qframe.bytesPerLine(0));
             glTexImage2D(GL_TEXTURE_2D, 0, GL_R8, w, h, 0, GL_RED, GL_UNSIGNED_BYTE, planeData[0]);
             glBindTexture(GL_TEXTURE_2D, _planeTexs[1]);
+            glPixelStorei(GL_UNPACK_ALIGNMENT, alignmentFromBytesPerLine(planeData[1], frame.qframe.bytesPerLine(1)));
+            glPixelStorei(GL_UNPACK_ROW_LENGTH, frame.qframe.bytesPerLine(1) / 2);
             glTexImage2D(GL_TEXTURE_2D, 0, GL_RG8, w / 2, h / 2, 0, GL_RG, GL_UNSIGNED_BYTE, planeData[1]);
             planeFormat = 4;
             planeCount = 2;
         } else if (frame.pixelFormat == QVideoFrameFormat::Format_P010
                 || frame.pixelFormat == QVideoFrameFormat::Format_P016) {
+            LOG_FIREHOSE("convertFrameToTexture: format p010/p016");
+            glPixelStorei(GL_UNPACK_ALIGNMENT, alignmentFromBytesPerLine(planeData[0], frame.qframe.bytesPerLine(0)));
+            glPixelStorei(GL_UNPACK_ROW_LENGTH, frame.qframe.bytesPerLine(0) / 2);
             glTexImage2D(GL_TEXTURE_2D, 0, GL_R16, w, h, 0, GL_RED, GL_UNSIGNED_SHORT, planeData[0]);
             glBindTexture(GL_TEXTURE_2D, _planeTexs[1]);
+            glPixelStorei(GL_UNPACK_ALIGNMENT, alignmentFromBytesPerLine(planeData[1], frame.qframe.bytesPerLine(1)));
+            glPixelStorei(GL_UNPACK_ROW_LENGTH, frame.qframe.bytesPerLine(1) / 4);
             glTexImage2D(GL_TEXTURE_2D, 0, GL_RG16, w / 2, h / 2, 0, GL_RG, GL_UNSIGNED_SHORT, planeData[1]);
             planeFormat = 4;
             planeCount = 2;
         } else if (frame.pixelFormat == QVideoFrameFormat::Format_Y8) {
+            glPixelStorei(GL_UNPACK_ALIGNMENT, alignmentFromBytesPerLine(planeData[0], frame.qframe.bytesPerLine(0)));
+            glPixelStorei(GL_UNPACK_ROW_LENGTH, frame.qframe.bytesPerLine(0));
+            LOG_FIREHOSE("convertFrameToTexture: format y8");
             glTexImage2D(GL_TEXTURE_2D, 0, GL_R8, w, h, 0, GL_RED, GL_UNSIGNED_BYTE, planeData[0]);
             planeFormat = 5;
             planeCount = 1;
         } else if (frame.pixelFormat == QVideoFrameFormat::Format_Y16) {
+            glPixelStorei(GL_UNPACK_ALIGNMENT, alignmentFromBytesPerLine(planeData[0], frame.qframe.bytesPerLine(0)));
+            glPixelStorei(GL_UNPACK_ROW_LENGTH, frame.qframe.bytesPerLine(0) / 2);
+            LOG_FIREHOSE("convertFrameToTexture: format y16");
             glTexImage2D(GL_TEXTURE_2D, 0, GL_R16, w, h, 0, GL_RED, GL_UNSIGNED_SHORT, planeData[0]);
             planeFormat = 5;
             planeCount = 1;
@@ -997,9 +1069,11 @@ void Bino::convertFrameToTexture(const VideoFrame& frame, unsigned int frameTex)
             std::exit(1);
         }
     }
+    glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
+    glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);
     // 2. Convert plane textures into linear RGB in the frame texture
     glBindTexture(GL_TEXTURE_2D, frameTex);
-    if (isGLES)
+    if (IsOpenGLES)
         glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB10_A2, w, h, 0, GL_RGBA, GL_UNSIGNED_INT_2_10_10_10_REV, nullptr);
     else
         glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA16, w, h, 0, GL_BGRA, GL_UNSIGNED_SHORT, nullptr);
@@ -1120,12 +1194,26 @@ void Bino::preRenderProcess(int screenWidth, int screenHeight,
 }
 
 void Bino::render(
+        const QVector3D& unitedScreenBottomLeft, const QVector3D& unitedScreenBottomRight, const QVector3D& unitedScreenTopLeft,
+        const QVector3D& intersectedScreenBottomLeft, const QVector3D& intersectedScreenBottomRight, const QVector3D& intersectedScreenTopLeft,
         const QMatrix4x4& projectionMatrix,
         const QMatrix4x4& orientationMatrix,
         const QMatrix4x4& viewMatrix,
         int view, // 0 = left, 1 = right
         int texWidth, int texHeight, unsigned int texture)
 {
+    // Update screen
+    switch (_screenType) {
+    case ScreenUnited:
+        _screen = Screen(unitedScreenBottomLeft, unitedScreenBottomRight, unitedScreenTopLeft);
+        break;
+    case ScreenIntersected:
+        _screen = Screen(intersectedScreenBottomLeft, intersectedScreenBottomRight, intersectedScreenTopLeft);
+        break;
+    case ScreenGeometry:
+        // do nothing
+        break;
+    }
     // Set up framebuffer object to render into
     glBindTexture(GL_TEXTURE_2D, _depthTex);
     glTexImage2D(GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT24, texWidth, texHeight,
@@ -1246,6 +1334,21 @@ void Bino::render(
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
     } else {
         glBindVertexArray(_screenVao);
+        if (_screenType == ScreenUnited || _screenType == ScreenIntersected) {
+            // the geometry might have changed; re-upload it
+            glBindBuffer(GL_ARRAY_BUFFER, _positionBuf);
+            glBufferData(GL_ARRAY_BUFFER, _screen.positions.size() * sizeof(float),
+                    _screen.positions.constData(), GL_STATIC_DRAW);
+            glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 0, 0);
+            glBindBuffer(GL_ARRAY_BUFFER, _texcoordBuf);
+            glBufferData(GL_ARRAY_BUFFER, _screen.texcoords.size() * sizeof(float),
+                    _screen.texcoords.constData(), GL_STATIC_DRAW);
+            glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, 0, 0);
+            glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, _indexBuf);
+            glBufferData(GL_ELEMENT_ARRAY_BUFFER,
+                    _screen.indices.length() * sizeof(unsigned int),
+                    _screen.indices.constData(), GL_STATIC_DRAW);
+        }
         glDrawElements(GL_TRIANGLES, _screen.indices.size(), GL_UNSIGNED_INT, 0);
     }
 }
@@ -1292,7 +1395,7 @@ void Bino::keyPressEvent(QKeyEvent* event)
         Playlist::instance()->next();
     } else if (event->key() == Qt::Key_P) {
         Playlist::instance()->prev();
-    } else if (event->matches(QKeySequence::FullScreen) || event->key() == Qt::Key_F) {
+    } else if (event->matches(QKeySequence::FullScreen) || event->key() == Qt::Key_F || event->key() == Qt::Key_F11) {
         emit toggleFullscreen();
     } else if (event->key() == Qt::Key_E || event->key() == Qt::Key_F7) {
         toggleSwapEyes();
