@@ -27,11 +27,15 @@
 #include <QAudioOutput>
 #include <QAudioInput>
 #include <QCamera>
-#include <QScreenCapture>
-#include <QWindowCapture>
 #include <QMediaPlayer>
 #include <QMediaCaptureSession>
 #include <QKeyEvent>
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
+# include <QScreenCapture>
+#endif
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 6, 0))
+# include <QWindowCapture>
+#endif
 
 #include "screen.hpp"
 #include "videosink.hpp"
@@ -42,6 +46,13 @@ class Bino : public QObject, QOpenGLExtraFunctions
 {
 Q_OBJECT
 
+public:
+    enum ScreenType {
+        ScreenUnited,           // global 2D united screen given by QVR
+        ScreenIntersected,      // global 2D intersected screen given by QVR
+        ScreenGeometry          // explicit geometry stored in _screen
+    };
+
 private:
     /* Data not directly relevant for rendering */
     bool _wantExit;
@@ -52,8 +63,12 @@ private:
     // for capturing audio/video:
     QAudioInput* _audioInput;
     QCamera* _videoInput;
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
     QScreenCapture* _screenInput;
+#endif
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 6, 0))
     QWindowCapture* _windowInput;
+#endif
     QMediaCaptureSession* _captureSession;
     // for rendering subtitles:
     QImage _subtitleImg;
@@ -63,6 +78,7 @@ private:
     SurroundMode _lastFrameSurroundMode;
 
     /* Static data for rendering, initialized on the main process */
+    ScreenType _screenType;
     Screen _screen;
 
     /* Static data for rendering, initialized in initProcess() */
@@ -75,7 +91,7 @@ private:
     unsigned int _frameTex;
     unsigned int _extFrameTex;
     unsigned int _subtitleTex;
-    unsigned int _screenVao;
+    unsigned int _screenVao, _positionBuf, _texcoordBuf, _indexBuf;
     QOpenGLShaderProgram _colorPrg;
     int _colorPrgPlaneFormat;
     bool _colorPrgYuvValueRangeSmall;
@@ -98,7 +114,7 @@ private:
     void convertFrameToTexture(const VideoFrame& frame, unsigned int frameTex);
 
 public:
-    Bino(const Screen& screen, bool swapEyes);
+    Bino(ScreenType screenType, const Screen& screen, bool swapEyes);
     virtual ~Bino();
 
     static Bino* instance();
@@ -112,14 +128,18 @@ public:
             bool withAudioInput,
             const QAudioDevice& audioInputDevice,
             const QCameraDevice& videoInputDevice);
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
     void startCaptureModeScreen(
             bool withAudioInput,
             const QAudioDevice& audioInputDevice,
             QScreen* screenInputDevice);
+#endif
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 6, 0))
     void startCaptureModeWindow(
             bool withAudioInput,
             const QAudioDevice& audioInputDevice,
             const QCapturableWindow& windowInputDevice);
+#endif
     void stopCaptureMode();
     bool playlistMode() const;
     bool captureMode() const;
@@ -178,6 +198,8 @@ public:
             float* frameDisplayAspectRatio = nullptr,
             bool* surround = nullptr);
     void render(
+            const QVector3D& unitedScreenBottomLeft, const QVector3D& unitedScreenBottomRight, const QVector3D& unitedScreenTopLeft,
+            const QVector3D& intersectedScreenBottomLeft, const QVector3D& intersectedScreenBottomRight, const QVector3D& intersectedScreenTopLeft,
             const QMatrix4x4& projectionMatrix,
             const QMatrix4x4& orientationMatrix,
             const QMatrix4x4& viewMatrix,
