@@ -20,6 +20,10 @@
 
 #include <QCommandLineParser>
 #include <QMediaDevices>
+#include <QGuiApplication>
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 6, 0))
+# include <QWindowCapture>
+#endif
 
 #include "commandinterpreter.hpp"
 #include "modes.hpp"
@@ -201,8 +205,12 @@ void CommandInterpreter::processNextCommand()
             bool ok;
             QList<QAudioDevice> audioInputDevices;
             QList<QCameraDevice> videoInputDevices;
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
             QList<QScreen*> screenInputDevices;
+#endif
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 6, 0))
             QList<QCapturableWindow> windowInputDevices;
+#endif
             int audioInputDeviceIndex = -1;
             int videoInputDeviceIndex = -1;
             int screenInputDeviceIndex = -1;
@@ -231,6 +239,7 @@ void CommandInterpreter::processNextCommand()
                     ok = false;
                 }
             }
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
             if (parser.isSet("screen-input")) {
                 screenInputDevices = QGuiApplication::screens();
                 int vi = parser.value("screen-input").toInt(&ok);
@@ -241,6 +250,8 @@ void CommandInterpreter::processNextCommand()
                     ok = false;
                 }
             }
+#endif
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 6, 0))
             if (parser.isSet("window-input")) {
                 windowInputDevices = QWindowCapture::capturableWindows();
                 int vi = parser.value("window-input").toInt(&ok);
@@ -251,6 +262,7 @@ void CommandInterpreter::processNextCommand()
                     ok = false;
                 }
             }
+#endif
             if (ok) {
                 if (screenInputDeviceIndex < 0 && windowInputDeviceIndex < 0) {
                     Bino::instance()->startCaptureModeCamera(audioInputDeviceIndex >= -1,
@@ -261,17 +273,21 @@ void CommandInterpreter::processNextCommand()
                             ? videoInputDevices[videoInputDeviceIndex]
                             : QMediaDevices::defaultVideoInput());
                 } else if (screenInputDeviceIndex >= 0) {
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
                     Bino::instance()->startCaptureModeScreen(audioInputDeviceIndex >= -1,
                             audioInputDeviceIndex >= 0
                             ? audioInputDevices[audioInputDeviceIndex]
                             : QMediaDevices::defaultAudioInput(),
                             screenInputDevices[screenInputDeviceIndex]);
+#endif
                 } else {
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 6, 0))
                     Bino::instance()->startCaptureModeWindow(audioInputDeviceIndex >= -1,
                             audioInputDeviceIndex >= 0
                             ? audioInputDevices[audioInputDeviceIndex]
                             : QMediaDevices::defaultAudioInput(),
                             windowInputDevices[windowInputDeviceIndex]);
+#endif
                 }
             }
         }
@@ -312,6 +328,14 @@ void CommandInterpreter::processNextCommand()
         Playlist::instance()->next();
     } else if (cmd == "playlist-prev") {
         Playlist::instance()->prev();
+    } else if (cmd.startsWith("playlist-wait ")) {
+        bool ok;
+        WaitMode waitMode = waitModeFromString(cmd.mid(14), &ok);
+        if (!ok) {
+            LOG_FATAL("%s", qPrintable(tr("Invalid argument in %1 line %2").arg(_file.fileName()).arg(_lineIndex)));
+        } else {
+            Playlist::instance()->setWaitMode(waitMode);
+        }
     } else if (cmd.startsWith("playlist-loop ")) {
         bool ok;
         LoopMode loopMode = loopModeFromString(cmd.mid(14), &ok);
