@@ -33,6 +33,9 @@
 #include <QComboBox>
 #include <QActionGroup>
 #include <QMimeData>
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 6, 0))
+# include <QWindowCapture>
+#endif
 
 #include "gui.hpp"
 #include "playlist.hpp"
@@ -64,14 +67,14 @@ Gui::Gui(OutputMode outputMode, bool fullscreen) :
 {
     setWindowTitle("Bino");
     QPixmap icon;
-    if (!icon.load(":aux/bino-logo-small.svg")) {
+    if (!icon.load(":res/bino-logo-small.svg")) {
         LOG_DEBUG("Gui: falling back to raster icon");
-        icon.load(":aux/bino-logo-small-512.png");
+        icon.load(":res/bino-logo-small-512.png");
     }
     setWindowIcon(QIcon(icon));
 
     QMenu* fileMenu = addBinoMenu(tr("&File"));
-    QAction* fileOpenAction = new QAction(tr("&Open File..."), this);
+    QAction* fileOpenAction = new QAction(tr("&Open File(s)..."), this);
     fileOpenAction->setShortcuts({ QKeySequence::Open });
     connect(fileOpenAction, SIGNAL(triggered()), this, SLOT(fileOpen()));
     addBinoAction(fileOpenAction, fileMenu);
@@ -128,6 +131,18 @@ Gui::Gui(OutputMode outputMode, bool fullscreen) :
     _playlistLoopActionGroup->addAction(playlistLoopAll)->setData(int(Loop_All));
     connect(playlistLoopAll, SIGNAL(triggered()), this, SLOT(playlistLoop()));
     addBinoAction(playlistLoopAll, playlistMenu);
+    playlistMenu->addSeparator();
+    _playlistWaitActionGroup = new QActionGroup(this);
+    QAction* playlistWaitOff = new QAction(waitModeToStringUI(Wait_Off), this);
+    playlistWaitOff->setCheckable(true);
+    _playlistWaitActionGroup->addAction(playlistWaitOff)->setData(int(Wait_Off));
+    connect(playlistWaitOff, SIGNAL(triggered()), this, SLOT(playlistWait()));
+    addBinoAction(playlistWaitOff, playlistMenu);
+    QAction* playlistWaitOn = new QAction(waitModeToStringUI(Wait_On), this);
+    playlistWaitOn->setCheckable(true);
+    _playlistWaitActionGroup->addAction(playlistWaitOn)->setData(int(Wait_On));
+    connect(playlistWaitOn, SIGNAL(triggered()), this, SLOT(playlistWait()));
+    addBinoAction(playlistWaitOn, playlistMenu);
 
     QMenu* threeDMenu = addBinoMenu(tr("&3D Modes"));
     _3dSurroundActionGroup = new QActionGroup(this);
@@ -418,7 +433,7 @@ Gui::Gui(OutputMode outputMode, bool fullscreen) :
 
     QMenu* viewMenu = addBinoMenu(tr("&View"));
     _viewToggleFullscreenAction = new QAction(tr("&Fullscreen"), this);
-    _viewToggleFullscreenAction->setShortcuts({ Qt::Key_F, QKeySequence::FullScreen });
+    _viewToggleFullscreenAction->setShortcuts({ Qt::Key_F11, Qt::Key_F, QKeySequence::FullScreen });
     _viewToggleFullscreenAction->setCheckable(true);
     connect(_viewToggleFullscreenAction, SIGNAL(triggered()), this, SLOT(viewToggleFullscreen()));
     addBinoAction(_viewToggleFullscreenAction, viewMenu);
@@ -462,19 +477,23 @@ Gui* Gui::instance()
 
 void Gui::fileOpen()
 {
-    QString name = QFileDialog::getOpenFileName(this);
-    if (!name.isEmpty()) {
-        QUrl url = QUrl::fromLocalFile(name);
-        MetaData metaData;
-        QString errMsg;
-        if (metaData.detectCached(url, &errMsg)) {
-            Bino::instance()->startPlaylistMode();
-            Playlist::instance()->clear();
-            Playlist::instance()->append(url);
-            Playlist::instance()->start();
-        } else {
-            QMessageBox::critical(this, tr("Error"), errMsg);
+    QStringList names = QFileDialog::getOpenFileNames(this);
+    if (!names.isEmpty()) {
+        bool playlistWasEmpty = Playlist::instance()->length() == 0;
+        Bino::instance()->startPlaylistMode();
+        for (int i = 0; i < names.size(); i++) {
+            QUrl url = QUrl::fromLocalFile(names[i]);
+            MetaData metaData;
+            QString errMsg;
+            if (metaData.detectCached(url, &errMsg)) {
+                Playlist::instance()->append(url);
+            } else {
+                QMessageBox::critical(this, tr("Error"), errMsg);
+            }
         }
+        Playlist::instance()->setWaitModeAuto();
+        if (playlistWasEmpty)
+            Playlist::instance()->next();
     }
 }
 
@@ -519,8 +538,12 @@ void Gui::fileOpenCamera()
     QList<QAudioDevice> audioOutputDevices = QMediaDevices::audioOutputs();
     QList<QAudioDevice> audioInputDevices = QMediaDevices::audioInputs();
     QList<QCameraDevice> videoInputDevices = QMediaDevices::videoInputs();
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
     QList<QScreen*> screenInputDevices = QGuiApplication::screens();
+#endif
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 6, 0))
     QList<QCapturableWindow> windowInputDevices = QWindowCapture::capturableWindows();
+#endif
     QGuiApplication::restoreOverrideCursor();
 
     QDialog *dialog = new QDialog(this);
@@ -534,25 +557,39 @@ void Gui::fileOpenCamera()
 
     QRadioButton *screenBtn = new QRadioButton(tr("Screen Input:"));
     QComboBox* screenBox = new QComboBox;
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
     for (int i = 0; i < screenInputDevices.size(); i++)
         screenBox->addItem(screenInputDevices[i]->name());
+#endif
 
     QRadioButton *windowBtn = new QRadioButton(tr("Window Input:"));
     QComboBox* windowBox = new QComboBox;
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 6, 0))
     for (int i = 0; i < windowInputDevices.size(); i++)
         windowBox->addItem(windowInputDevices[i].description());
+#endif
 
     videoBtn->setChecked(true);
     screenBtn->setChecked(false);
     windowBtn->setChecked(false);
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
     if (screenInputDevices.size() == 0) {
         screenBtn->setEnabled(false);
         screenBox->setEnabled(false);
     }
+#else
+    screenBtn->setEnabled(false);
+    screenBox->setEnabled(false);
+#endif
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 6, 0))
     if (windowInputDevices.size() == 0) {
         windowBtn->setEnabled(false);
         windowBox->setEnabled(false);
     }
+#else
+    windowBtn->setEnabled(false);
+    windowBox->setEnabled(false);
+#endif
 
     QLabel *audioLabel = new QLabel(tr("Audio Input:"));
     QComboBox* audioBox = new QComboBox;
@@ -594,19 +631,23 @@ void Gui::fileOpenCamera()
                     ? videoInputDevices[videoInputDeviceIndex]
                     : QMediaDevices::defaultVideoInput());
         } else if (screenBtn->isChecked()) {
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
             int screenInputDeviceIndex = screenBox->currentIndex();
             Bino::instance()->startCaptureModeScreen(audioInputDeviceIndex >= -1,
                     audioInputDeviceIndex >= 0
                     ? audioInputDevices[audioInputDeviceIndex]
                     : QMediaDevices::defaultAudioInput(),
                     screenInputDevices[screenInputDeviceIndex]);
+#endif
         } else if (windowBtn->isChecked()) {
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 6, 0))
             int windowInputDeviceIndex = windowBox->currentIndex();
             Bino::instance()->startCaptureModeWindow(audioInputDeviceIndex >= -1,
                     audioInputDeviceIndex >= 0
                     ? audioInputDevices[audioInputDeviceIndex]
                     : QMediaDevices::defaultAudioInput(),
                     windowInputDevices[windowInputDeviceIndex]);
+#endif
         }
     }
 }
@@ -685,6 +726,13 @@ void Gui::playlistLoop()
         Playlist::instance()->setLoopMode(static_cast<LoopMode>(a->data().toInt()));
 }
 
+void Gui::playlistWait()
+{
+    QAction* a = _playlistWaitActionGroup->checkedAction();
+    if (a)
+        Playlist::instance()->setWaitMode(static_cast<WaitMode>(a->data().toInt()));
+}
+
 void Gui::threeDSurround()
 {
     QAction* a = _3dSurroundActionGroup->checkedAction();
@@ -883,6 +931,12 @@ void Gui::updateActions()
         a->setChecked(a->data().toInt() == int(loopMode));
     }
 
+    WaitMode waitMode = Playlist::instance()->waitMode();
+    for (int i = 0; i < _playlistWaitActionGroup->actions().size(); i++) {
+        QAction* a = _playlistWaitActionGroup->actions()[i];
+        a->setChecked(a->data().toInt() == int(waitMode));
+    }
+
     SurroundMode surroundMode = Bino::instance()->assumeSurroundMode();
     for (int i = 0; i < _3dSurroundActionGroup->actions().size(); i++) {
         QAction* a = _3dSurroundActionGroup->actions()[i];
