@@ -40,9 +40,14 @@
 #include <QAudioOutput>
 #include <QAudioInput>
 #include <QCamera>
-#include <QWindowCapture>
 #include <QSurfaceFormat>
 #include <QOpenGLContext>
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 6, 0))
+# include <QWindowCapture>
+#endif
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
+# include <QtProcessorDetection>
+#endif
 
 #ifdef WITH_QVR
 #  include <qvr/manager.hpp>
@@ -58,6 +63,7 @@
 #include "gui.hpp"
 #include "commandinterpreter.hpp"
 #include "modes.hpp"
+#include "tools.hpp"
 #include "bino.hpp"
 
 
@@ -133,9 +139,10 @@ int main(int argc, char* argv[])
     parser.addOption({ "vr",
             QCommandLineParser::tr("Start in VR mode instead of GUI mode.")});
     parser.addOption({ "vr-screen",
-            QCommandLineParser::tr("Set VR screen geometry, either as a comma-separated list of "
-            "nine values representing three 3D coordinates that define a planar screen (bottom left, bottom right, top left) "
-            "or as a name of an OBJ file that contains the screen geometry with texture coordinates."),
+            QCommandLineParser::tr("Set VR screen geometry, either as the special values 'united' or 'intersected', or "
+            "as a comma-separated list of nine values representing three 3D coordinates that define a planar screen (bottom left, bottom right, top left), "
+            "or as a an aspect ratio followed by the name of an OBJ file that contains the screen geometry with texture coordinates "
+            "(example: '16:9,myscreen.obj')."),
             "screen" });
     parser.addOption({ "capture",
             QCommandLineParser::tr("Capture audio/video input from microphone and camera/screen/window.") });
@@ -185,6 +192,9 @@ int main(int argc, char* argv[])
     parser.addOption({ { "l", "loop" },
             QCommandLineParser::tr("Set loop mode (%1).").arg("off, one, all"),
             "mode" });
+    parser.addOption({ { "w", "wait" },
+            QCommandLineParser::tr("Set wait mode (%1).").arg("off, on"),
+            "mode" });
     parser.addOption({ { "i", "input" },
             QCommandLineParser::tr("Set input mode (%1).").arg("mono, "
             "top-bottom, top-bottom-half, bottom-top, bottom-top-half, "
@@ -275,8 +285,12 @@ int main(int argc, char* argv[])
     QList<QAudioDevice> audioOutputDevices;
     QList<QAudioDevice> audioInputDevices;
     QList<QCameraDevice> videoInputDevices;
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
     QList<QScreen*> screenInputDevices;
+#endif
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 6, 0))
     QList<QCapturableWindow> windowInputDevices;
+#endif
 
     // List devices
     bool deviceListRequested = false;
@@ -311,6 +325,7 @@ int main(int argc, char* argv[])
         deviceListRequested = true;
     }
     if (parser.isSet("list-screen-inputs")) {
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
         screenInputDevices = QGuiApplication::screens();
         if (screenInputDevices.size() == 0) {
             LOG_REQUESTED("%s", qPrintable(QCommandLineParser::tr("No screen inputs available.")));
@@ -318,9 +333,13 @@ int main(int argc, char* argv[])
             for (qsizetype i = 0; i < screenInputDevices.size(); i++)
                 LOG_REQUESTED("%s", qPrintable(QCommandLineParser::tr("Screen input %1: %2").arg(i).arg(screenInputDevices[i]->name())));
         }
+#else
+        LOG_REQUESTED("%s", qPrintable(QCommandLineParser::tr("No screen inputs available.")));
+#endif
         deviceListRequested = true;
     }
     if (parser.isSet("list-window-inputs")) {
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 6, 0))
         windowInputDevices = QWindowCapture::capturableWindows();
         if (windowInputDevices.size() == 0) {
             LOG_REQUESTED("%s", qPrintable(QCommandLineParser::tr("No window inputs available.")));
@@ -328,6 +347,9 @@ int main(int argc, char* argv[])
             for (qsizetype i = 0; i < windowInputDevices.size(); i++)
                 LOG_REQUESTED("%s", qPrintable(QCommandLineParser::tr("Window input %1: %2").arg(i).arg(windowInputDevices[i].description())));
         }
+#else
+        LOG_REQUESTED("%s", qPrintable(QCommandLineParser::tr("No window inputs available.")));
+#endif
         deviceListRequested = true;
     }
     if (deviceListRequested) {
@@ -380,6 +402,7 @@ int main(int argc, char* argv[])
             }
         }
         if (parser.isSet("screen-input")) {
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
             if (screenInputDevices.size() == 0)
                 screenInputDevices = QGuiApplication::screens();
             int vi = parser.value("screen-input").toInt(&ok);
@@ -389,8 +412,13 @@ int main(int argc, char* argv[])
                 LOG_FATAL("%s", qPrintable(QCommandLineParser::tr("Invalid argument for option %1").arg("--screen-input")));
                 return 1;
             }
+#else
+            LOG_FATAL("%s", qPrintable(QCommandLineParser::tr("Invalid argument for option %1").arg("--screen-input")));
+            return 1;
+#endif
         }
         if (parser.isSet("window-input")) {
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 6, 0))
             if (windowInputDevices.size() == 0)
                 windowInputDevices = QWindowCapture::capturableWindows();
             int vi = parser.value("window-input").toInt(&ok);
@@ -400,6 +428,10 @@ int main(int argc, char* argv[])
                 LOG_FATAL("%s", qPrintable(QCommandLineParser::tr("Invalid argument for option %1").arg("--window-input")));
                 return 1;
             }
+#else
+            LOG_FATAL("%s", qPrintable(QCommandLineParser::tr("Invalid argument for option %1").arg("--window-input")));
+            return 1;
+#endif
         }
     }
 
@@ -437,6 +469,15 @@ int main(int argc, char* argv[])
         }
         playlist.setLoopMode(loopMode);
     }
+    if (parser.isSet("wait")) {
+        bool ok;
+        WaitMode waitMode = waitModeFromString(parser.value("wait"), &ok);
+        if (!ok) {
+            LOG_FATAL("%s", qPrintable(QCommandLineParser::tr("Invalid argument for option %1").arg("--wait")));
+            return 1;
+        }
+        playlist.setWaitMode(waitMode);
+    }
     int videoTrack = PlaylistEntry::DefaultTrack;
     int audioTrack = PlaylistEntry::DefaultTrack;
     int subtitleTrack = PlaylistEntry::DefaultTrack;
@@ -500,6 +541,9 @@ int main(int argc, char* argv[])
             return 1;
         }
     }
+    if (!parser.isSet("wait")) {
+        playlist.setWaitModeAuto();
+    }
     if (parser.isSet("capture") && playlist.length() > 0) {
         LOG_FATAL("%s", qPrintable(QCommandLineParser::tr("Cannot capture and play URL at the same time.")));
         return 1;
@@ -546,30 +590,37 @@ int main(int argc, char* argv[])
     }
 
     // Handle the VR Screen
-    Screen screen;
+    Bino::ScreenType screenType = Bino::ScreenGeometry;
+    Screen screen; // default screen for non-VR mode
     if (parser.isSet("vr")) {
         float screenCenterHeight = 1.76f - 0.15f; // does not matter, never used
 #ifdef WITH_QVR
         screenCenterHeight = QVRObserverConfig::defaultEyeHeight;
 #endif
-        screen = Screen(
+        screen = Screen( // default screen for VR mode
                 QVector3D(-16.0f / 9.0f, -1.0f + screenCenterHeight, -8.0f),
                 QVector3D(+16.0f / 9.0f, -1.0f + screenCenterHeight, -8.0f),
                 QVector3D(-16.0f / 9.0f, +1.0f + screenCenterHeight, -8.0f));
         if (parser.isSet("vr-screen")) {
             QStringList paramList = parser.value("vr-screen").split(',');
             float values[9];
-            if (paramList.length() == 9
+            if (parser.value("vr-screen") == "united") {
+                screenType = Bino::ScreenUnited;
+            } else if (parser.value("vr-screen") == "intersected") {
+                screenType = Bino::ScreenIntersected;
+            } else if (paramList.length() == 9
                     && 9 == std::sscanf(qPrintable(parser.value("vr-screen")),
                         "%f,%f,%f,%f,%f,%f,%f,%f,%f",
                         values + 0, values + 1, values + 2,
                         values + 3, values + 4, values + 5,
                         values + 6, values + 7, values + 8)) {
+                screenType = Bino::ScreenGeometry;
                 screen = Screen(
                         QVector3D(values[0], values[1], values[2]),
                         QVector3D(values[3], values[4], values[5]),
                         QVector3D(values[6], values[7], values[8]));
             } else if (paramList.length() == 2) {
+                screenType = Bino::ScreenGeometry;
                 float ar;
                 float ar2[2];
                 if (2 == std::sscanf(qPrintable(paramList[0]), "%f:%f", ar2 + 0, ar2 + 1)) {
@@ -608,11 +659,21 @@ int main(int argc, char* argv[])
     format.setBlueBufferSize(10);
     format.setAlphaBufferSize(0);
     format.setStencilBufferSize(0);
-    if (parser.isSet("opengles"))
+    bool wantOpenGLES = parser.isSet("opengles");
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
+# if defined Q_PROCESSOR_ARM
+    wantOpenGLES = true;
+# endif
+#else
+# if defined(__arm__) || defined(__TARGET_ARCH_ARM) || defined(_M_ARM) || defined(_M_ARM64) || defined(__aarch64__) || defined(__ARM64__)
+    wantOpenGLES = true;
+# endif
+#endif
+    if (wantOpenGLES)
         format.setRenderableType(QSurfaceFormat::OpenGLES);
-    if (QOpenGLContext::openGLModuleType() == QOpenGLContext::LibGLES
-            || format.renderableType() == QSurfaceFormat::OpenGLES) {
-        format.setVersion(3, 2);
+    initializeIsOpenGLES(format);
+    if (IsOpenGLES) {
+        format.setVersion(3, 1);
     } else {
         format.setProfile(QSurfaceFormat::CoreProfile);
         format.setVersion(3, 3);
@@ -633,7 +694,7 @@ int main(int argc, char* argv[])
     QSurfaceFormat::setDefaultFormat(format);
 
     // Initialize Bino (in VR mode: only from the main process!)
-    Bino bino(screen, parser.isSet("swap-eyes"));
+    Bino bino(screenType, screen, parser.isSet("swap-eyes"));
     if (guiMode || !vrChildProcess) {
         bino.initializeOutput(audioOutputDeviceIndex >= 0
                 ? audioOutputDevices[audioOutputDeviceIndex]
@@ -648,17 +709,21 @@ int main(int argc, char* argv[])
                         ? videoInputDevices[videoInputDeviceIndex]
                         : QMediaDevices::defaultVideoInput());
             } else if (screenInputDeviceIndex >= 0) {
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 5, 0))
                 bino.startCaptureModeScreen(audioInputDeviceIndex >= -1,
                         audioInputDeviceIndex >= 0
                         ? audioInputDevices[audioInputDeviceIndex]
                         : QMediaDevices::defaultAudioInput(),
                         screenInputDevices[screenInputDeviceIndex]);
+#endif
             } else {
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 6, 0))
                 bino.startCaptureModeWindow(audioInputDeviceIndex >= -1,
                         audioInputDeviceIndex >= 0
                         ? audioInputDevices[audioInputDeviceIndex]
                         : QMediaDevices::defaultAudioInput(),
                         windowInputDevices[windowInputDeviceIndex]);
+#endif
             }
         } else {
             bino.startPlaylistMode();
