@@ -1,7 +1,7 @@
 /*
  * This file is part of Bino, a 3D video player.
  *
- * Copyright (C) 2022, 2023
+ * Copyright (C) 2022, 2023, 2024
  * Martin Lambers <marlam@marlam.de>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -86,9 +86,7 @@ QSize Widget::sizeHint() const
 
 void Widget::initializeGL()
 {
-    bool contextIsOk = (context()->isValid()
-            && (context()->format().majorVersion() > 3
-                || (context()->format().majorVersion() == 3 && context()->format().minorVersion() >= 2)));
+    bool contextIsOk = (context()->isValid() && context()->format().majorVersion() >= 3);
     if (!contextIsOk) {
         LOG_FATAL("%s", qPrintable(tr("Insufficient OpenGL capabilities.")));
         QMessageBox::critical(this, tr("Error"), tr("Insufficient OpenGL capabilities."));
@@ -100,13 +98,12 @@ void Widget::initializeGL()
         std::exit(1);
     }
 
-    bool isGLES = QOpenGLContext::currentContext()->isOpenGLES();
     bool haveAnisotropicFiltering = checkTextureAnisotropicFilterAvailability();
     initializeOpenGLFunctions();
     bool isCoreProfile = (QOpenGLContext::currentContext()->format().profile() == QSurfaceFormat::CoreProfile);
 
-    QString variantString = isGLES ? "OpenGL ES" : "OpenGL";
-    if (!isGLES)
+    QString variantString = IsOpenGLES ? "OpenGL ES" : "OpenGL";
+    if (!IsOpenGLES)
         variantString += isCoreProfile ? " core profile" : " compatibility profile";
     LOG_INFO("OpenGL Variant:      %s", qPrintable(variantString));
     LOG_INFO("OpenGL Version:      %s", getOpenGLString(this, GL_VERSION));
@@ -120,7 +117,7 @@ void Widget::initializeGL()
     for (int i = 0; i < 2; i++) {
         glBindTexture(GL_TEXTURE_2D, _viewTex[i]);
         unsigned char nullBytes[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
-        if (isGLES)
+        if (IsOpenGLES)
             glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB10_A2, 1, 1, 0, GL_RGBA, GL_UNSIGNED_INT_2_10_10_10_REV, nullBytes);
         else
             glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA16, 1, 1, 0, GL_RGBA, GL_UNSIGNED_SHORT, nullBytes);
@@ -183,13 +180,12 @@ void Widget::rebuildDisplayPrgIfNecessary(OutputMode outputMode)
         return;
 
     LOG_DEBUG("rebuilding display program for output mode %s", outputModeToString(outputMode));
-    bool isGLES = QOpenGLContext::currentContext()->isOpenGLES();
     QString vertexShaderSource = readFile(":src/shader-display.vert.glsl");
     QString fragmentShaderSource = readFile(":src/shader-display.frag.glsl");
     fragmentShaderSource.replace("$OUTPUT_MODE", QString::number(int(outputMode)));
-    if (isGLES) {
-        vertexShaderSource.prepend("#version 320 es\n");
-        fragmentShaderSource.prepend("#version 320 es\n"
+    if (IsOpenGLES) {
+        vertexShaderSource.prepend("#version 300 es\n");
+        fragmentShaderSource.prepend("#version 300 es\n"
                 "precision mediump float;\n");
     } else {
         vertexShaderSource.prepend("#version 330\n");
@@ -204,8 +200,6 @@ void Widget::rebuildDisplayPrgIfNecessary(OutputMode outputMode)
 
 void Widget::paintGL()
 {
-    bool isGLES = QOpenGLContext::currentContext()->isOpenGLES();
-
     // Support for HighDPI output
     int width = _width * devicePixelRatioF();
     int height = _height * devicePixelRatioF();
@@ -274,7 +268,7 @@ void Widget::paintGL()
         // prepare view texture
         glBindTexture(GL_TEXTURE_2D, _viewTex[v]);
         if (_viewTexWidth[v] != viewWidth || _viewTexHeight[v] != viewHeight) {
-            if (isGLES)
+            if (IsOpenGLES)
                 glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB10_A2, viewWidth, viewHeight, 0, GL_RGBA, GL_UNSIGNED_INT_2_10_10_10_REV, nullptr);
             else
                 glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB16, viewWidth, viewHeight, 0, GL_RGBA, GL_UNSIGNED_SHORT, nullptr);
@@ -299,7 +293,9 @@ void Widget::paintGL()
                     (_surroundHorizontalAngleBase + _surroundHorizontalAngleCurrent), 0.0f);
             orientationMatrix.rotate(orientation.inverted());
         }
-        Bino::instance()->render(projectionMatrix, orientationMatrix, viewMatrix, v, viewWidth, viewHeight, _viewTex[v]);
+        Bino::instance()->render(
+                QVector3D(), QVector3D(), QVector3D(), QVector3D(), QVector3D(), QVector3D(),
+                projectionMatrix, orientationMatrix, viewMatrix, v, viewWidth, viewHeight, _viewTex[v]);
         // generate mipmaps for the view texture
         glBindTexture(GL_TEXTURE_2D, _viewTex[v]);
         glGenerateMipmap(GL_TEXTURE_2D);
